
'''
 The following functions are defined by the user and inputs into the genetic algorithm. These are default values which are later set to the user's respective functions.
'''



## Map objects to real numbers based on fitness criteria
def cost(object):
    return 1

## Establish a bijection between DNA-bitstrings and objects
def decoder(DNA):
    return 1
